"use strict";
/**
 * copy-ratings.js
 * Copies match ratings (HT-ML style)
 * @author spambot, ryanli
 */

Foxtrick.util.module.register({
	MODULE_NAME : "CopyRatings",
	MODULE_CATEGORY : Foxtrick.moduleCategories.MATCHES,
	PAGES : new Array('match'),

	NICE : 1, // after MatchReportFormat.

	CSS : Foxtrick.InternalPath + "resources/css/copy-ratings.css",

	run : function(doc) {
		if (Foxtrick.Pages.Match.isPrematch(doc))
			return;

		var table = Foxtrick.Pages.Match.getRatingsTable(doc);
		if (!table)
			return;

		var createRatings = function(place, teams) {
			try {
				if (place == "table")
					var insertBefore = doc.getElementById('mainBody').getElementsByTagName('h2')[0].parentNode;
				else if (place == "box")
					var insertBefore = doc.getElementById('foxtrick_addactionsbox_parentDiv');
				else {
					var insertBefore = doc.getElementsByTagName('h1')[0];
				}
				var team1 = (teams == "both") || (teams == "home");
				var team2 = (teams == "both") || (teams == "away");

				var _d = Foxtrickl10n.getString("foxtrick.matchdetail.defence")+':';
				var _m = Foxtrickl10n.getString("foxtrick.matchdetail.midfield")+':';
				var _a = Foxtrickl10n.getString("foxtrick.matchdetail.attack")+':';
				var _t = Foxtrickl10n.getString("foxtrick.matchdetail.total")+':';

				var headder = doc.getElementsByTagName('h1')[0].innerHTML;
				headder=Foxtrick.trim(headder);
				var start = Foxtrick.strrpos(headder, '<span>(') +7;
				var end = Foxtrick.strrpos(headder, ')</span>');

				var matchlink=doc.getElementById('mainWrapper').getElementsByTagName('h2')[0].getElementsByTagName('a')[0];
				var gameid = Foxtrick.util.id.getMatchIdFromUrl(matchlink.href);// headder.substr(start, end-start);

				start = Foxtrick.strrpos(headder, ' - ');
				var gameresult_h = Foxtrick.trim(headder.substr(start-2, 2));
				var gameresult_a = Foxtrick.trim(headder.substr(start+3, 2));

				var ad = '\n[table]\n';
				var table = Foxtrick.Pages.Match.getRatingsTable(doc).cloneNode(true);
				for (var row=0; row<table.rows.length; ++row) {
					if(!team1 && table.rows[row].cells.length>=2) table.rows[row].cells[1].innerHTML='###';
					if(!team2 && table.rows[row].cells.length>=3) table.rows[row].cells[2].innerHTML='###';
				}

				var youth = '';
				if (matchlink.href.search('isYouth=True')!=-1) youth = 'youth';

				// head row
				ad += '[tr]\n\n[th]';
				if ((table.rows[0].cells[0])) {
					ad += '['+ youth + 'matchid=' + gameid + ']';
				}
				ad += '[/th]\n[th]';
				if (team1 && table.rows[0].cells[1]) {
					var teamlink = table.rows[0].cells[1].getElementsByTagName('a')[0];
					if (teamlink)
						ad += teamlink.innerHTML + ((team2==true)?(' - ' + gameresult_h):'') + '[br]['+youth+'teamid='+Foxtrick.util.id.getTeamIdFromUrl(teamlink.href)+']';
				}
				if (team1 && team2)
					ad += '[/th]\n[th]';
				if (team2 && table.rows[0].cells[2]) {
					var teamlink = table.rows[0].cells[2].getElementsByTagName('a')[0];
					if (teamlink)
						ad += teamlink.innerHTML + ((team1==true)?(' - ' + gameresult_a):'') + '[br]['+youth+'teamid='+Foxtrick.util.id.getTeamIdFromUrl(teamlink.href)+']';
				}
				ad += '[/th]\n\n[/tr]\n';

				for (var row = 1; row < table.rows.length; ++row) {
					try {
						ad += '[tr]\n\n[th]';
						if (table.rows[row].cells[0]) {
							ad += table.rows[row].cells[0].textContent;
						}
						ad += '[/th]\n[td]';
						if (team1) {
							if (table.rows[row].cells[1]) {
								ad += table.rows[row].cells[1].textContent.replace(_d, '[br]'+_d).replace(_m, '[br]'+_m).replace(_a, '[br]'+_a).replace(_t, '[br]'+_t);
							}
							if (table.rows[row].cells[3]) {
								//ad += ' (' + table.rows[row].cells[3].textContent.replace(',','.') + ')';
							}
						}
						if (team1 && team2)
							ad += '[/td]\n[td]';
						if (team2) {
							if (table.rows[row].cells[2]) {
								ad += table.rows[row].cells[2].textContent.replace(_d, '[br]'+_d).replace(_m, '[br]'+_m).replace(_a, '[br]'+_a).replace(_t, '[br]'+_t);
							}
							if (table.rows[row].cells[4]) {
								//ad += ' (' + table.rows[row].cells[4].textContent + ')';
							}
						}
						ad += '[/td]\n\n[/tr]\n';
					} catch (e) {Foxtrick.log(e)}
				}
				ad = ad.replace(/\[td\]###\[\/td\]/gi,'');
				ad += '\n[/table]\n';

				// copy htms prediction. 
				if (team1 && team2) {
 					var htmsMatchDivId = doc.getElementById('htmsMatchDivId');
					if (htmsMatchDivId) {
						ad += Foxtrick.util.module.get('HTMSPrediction').copy(htmsMatchDivId);
					}
				}
				
				Foxtrick.copyStringToClipboard(ad);
				var note = Foxtrick.util.note.add(doc, insertBefore, "ft-ratings-copy-note", Foxtrickl10n.getString("CopyRatings.copied"), null, true);
			}
			catch (e) {
				Foxtrick.log(e);
			}
		};

		// Copy links inside the table
		var tableHeader = table.parentNode.parentNode.getElementsByClassName("tblBox")[0];
		var homeHeader = table.getElementsByTagName("th")[1];
		var awayHeader = table.getElementsByTagName("th")[2];

		var copyBoth = doc.createElement("span");
		copyBoth.className = "ft_copy_rating";
		copyBoth.appendChild(doc.createTextNode(Foxtrickl10n.getString("Copy")));
		copyBoth.setAttribute("teams", "both");
		copyBoth.setAttribute("place", "table");
		Foxtrick.listen(copyBoth, "click", function() { createRatings("table", "both"); }, false);
		tableHeader.appendChild(copyBoth);

		var copyHome = doc.createElement("span");
		copyHome.className = "ft_copy_rating";
		copyHome.appendChild(doc.createTextNode("(" + Foxtrickl10n.getString("Copy") + ")"));
		copyHome.setAttribute("teams", "home");
		copyHome.setAttribute("place", "table");
		Foxtrick.listen(copyHome, "click", function() { createRatings("table", "home"); }, false);
		homeHeader.appendChild(copyHome);

		var copyAway = doc.createElement("span");
		copyAway.className = "ft_copy_rating";
		copyAway.appendChild(doc.createTextNode("(" + Foxtrickl10n.getString("Copy") + ")"));
		copyAway.setAttribute("teams", "away");
		copyAway.setAttribute("place", "table");
		Foxtrick.listen(copyAway, "click", function() { createRatings("table", "away"); }, false);
		awayHeader.appendChild(copyAway);

		var button = Foxtrick.util.copyButton.add(doc,
			Foxtrickl10n.getString("CopyRatings.both"));
		if (button) {
			button.title='';
			Foxtrick.addClass(button, "ft-copy-ratings ft-pop-up-container");

			var versions=['both','home','away'];
			var list = doc.createElement("ul");
			list.className = "ft-pop";
			for (var j=0; j<versions.length; ++j) {
				var item = doc.createElement("li");
				var link = doc.createElement("span");
				Foxtrick.listen(link, "click", (function(team) {
					// to keep team variable here
					return function() { createRatings("box", team); };
				})(versions[j]), false);
				link.setAttribute("teams", versions[j]);
				link.textContent = Foxtrickl10n.getString("CopyRatings."+versions[j]);
				item.appendChild(link);
				list.appendChild(item);
			}
			button.appendChild(list);
		}
	}
});
